% LADCP data processing
% 
% LDEO Martin Visbeck March 2004
%
% Main process control file PART: I
% calls PRESLOVE.M at the end

clear d dr 

% rerun with same settings
if exist('pin')==1
 yn=input('found original p-structure (saved as pin). You you want to use that? [y]/n ','s');
 if yn=='n' | yn=='N';
  disp(' use current p')
 else
  disp(' copy pin > p')
  p=pin;
 end
else
 % save input
 pin=p;
end

if exist('psin')==1
 yn=input('found original ps-structure (saved as psin). You you want to use that? [y]/n ','s');
 if yn=='n' | yn=='N';
  disp(' use current ps')
 else
  disp(' copy psin > ps')
  ps=psin;
 end
else
 % save input
 if exist('ps')~=1
  default
 end
 psin=ps;
end

% Start Processing the data
%
% first load default parameter you might want to set
%
tic;
default

% open all windows
openwindows

% check if output files are wanted, open log file
if length(f.res)>1
 if exist([f.res,'.log'])==exist('loadrdi.m')
  eval(['delete ',f.res,'.log'])
 end
 diary([f.res,'.log'])
 diary on
end

% save the software version used to log file
disp(p.software)

% LOAD RDI BB-raw data
%  this is a rather complex set of functions
%  1) load raw data
%     apply magentic deviation if given
%  2) merge down-up data
%  3) do some fist order error check
 [d,p]=loadrdi(f,p); 

disp(['==> load data took ',int2str(toc),' seconds'])
if p.debug==9, keyboard, elseif p.debug>1, pause, end
toco=toc;

% fix problems with switched beams on instrument
if existf(p,'beam_switch')==1
 [d,p]=switchbeams(d,p);
 if p.debug==9, keyboard, elseif p.debug>1, pause, end
end

% fix problems with a compass
if p.fix_compass>0
 [d,p]=fixcompass(d,p);
 if p.debug==9, keyboard, elseif p.debug>1, pause, end
end

% look for log files to parse out serial numbers of instruments and save their content
p=getserial(f,p);
if p.debug==9, keyboard, elseif p.debug>1, pause, end

% LOAD other data useful for processing
% 
% This version of the software assume that you keep those files local
% in your working direcotory
% Look for:
%   a) Navigation time series
%   b) CTD processed data to get soundspeed and N^2
%   c) CTD time series data to get depth of instrument
%
% get GPS navigation data and  apply magnetic deviation if not done already
% We provide more than one version to support different file formats
% 

if length(f.nav)>1 & exist('loadnav')==exist('loadrdi')
  [d,p]=loadnav(f,d,p);
  if p.debug==9, keyboard, elseif p.debug>1, pause, end
else
  d.slon=NaN*d.time_jul;
  d.slat=d.slon;
end

%  Check if hbot values are mostly ==0
%
%  some RDI instruments seem to have trouble reporting the distance of the bottom
%  despite giving reasonable bottom track values.
%  If this problem is diagnosed we make the distance ourselves.
%
ii1=sum(isfinite(d.hbot));
ii0=sum(d.hbot==0);
p.hbot_0=ii0/(ii1+1)*100;

if p.hbot_0>80 
 p.bottomdist=1;
 disp([' WARNING found ',int2str(p.hbot_0),'% of  hbot=0  WARNING'])
end

[d,p]=getbtrack(d,p);  
if p.debug==9, keyboard, elseif p.debug>1, pause, end


%  get processed ctd profile data
%  We provide more than one version to support different file formats
% 
if length(f.ctdprof)>1 & exist('loadctdprof')==exist('loadrdi')
  [d,p]=loadctdprof(f,d,p);
  if p.debug==9, keyboard, elseif p.debug>1, pause, end
end

%  get ctd time series data
%  We provide more than one version to support different file formats
% 
if length(f.ctd)>1 & exist('loadctd')==exist('loadrdi')
  [d,p]=loadctd(f,d,p);
  if p.debug==9, keyboard, elseif p.debug>1, pause, end
end

% ==================================================================================
% have read most of the data and now process them
% ==================================================================================
%
%  Find depth and bottom and surface using ADCP data 
if p.getdepth==2
 [d,p]=getdpthi(d,p);
else
 [d,p]=getdpth(d,p);
end
if p.debug==9, keyboard, elseif p.debug>1, pause, end

disp(['==> get depth took ',int2str(toc-toco),' seconds'])
toco=toc;

% Plot a summary plot of the raw data
  figure(2), clf
  p=plotraw(d,p);
  pause(.01)

  if p.debug==9, keyboard, elseif p.debug>1, pause, end

% ==================================================================================
% have time series of instrument depths
% ==================================================================================
%  call the second processing command file.
%  this allows to "reprocess" data without loading and finding the depth again
disp(' LAPROC finished, call PRESOLVE ')
if p.debug==9, keyboard, elseif p.debug>1, pause, end

presolve
