%%
%  It is assumed that LAPROC.M ran up to this point
%
% Compute Super Ensemble from raw data to reduce the dimension
% of the problem
% also
%    1) rotate down and up instrument together
%    2) check bottom track data
%    3) check for large tilt
%    4) compute tilt and heading offset between instruments
%    5) read SADCP data if available
% 

% 
% this allow to apply a tilt correction to the instruments
%

if length(p.tiltcor)>1
 pd.dpit=p.tiltcor(1);
 pd.drol=p.tiltcor(2);
 d=uvwrot(d,pd,1);
 if p.debug==9, keyboard, elseif p.debug>1, pause, end
end

if length(p.tiltcor)>2
 pu.dpit=p.tiltcor(3);
 pu.drol=p.tiltcor(4);
 d=uvwrot(d,pu,0);
 if p.debug==9, keyboard, elseif p.debug>1, pause, end
end

%
% call the main function that makes the super ensembles 
%
[di,p,d]=prepinv(d,p);
if p.debug==9, keyboard, elseif p.debug>1, pause, end


% Reduce scatter by successively removing 1% of the data
%  in oder to do that we need a first solution
%
if ps.outlier>0 | p.offsetup2down>0
   diary off
   if exist('loadsadcp')==exist('loadrdi') 
    [di,p]=loadsadcp(f,di,p);
    if p.debug==9, keyboard, elseif p.debug>1, pause, end
   end
   dino=di;
   lanarrow
   if p.debug==9, keyboard, elseif p.debug>1, pause, end
   diary on
end

%
% onces we have a first guess profile we recompute the super ensemble
%
if (p.offsetup2down>0 & length(d.izu)>0)
 diary off
 [di,p,d]=prepinv(d,p,dr);
 if p.debug==9, keyboard, elseif p.debug>1, pause, end
 diary on
end
 
% 
% load SADCP data for final processing
%
if exist('loadsadcp')==exist('loadrdi') 
 di=loadsadcp(f,di,p);
 if p.debug==9, keyboard, elseif p.debug>1, pause, end
end

%
% ==============================================================
%  all data are prepared for the inversion 
% ==============================================================
%
 
disp(['==> prep inversion took ',int2str(toc-toco),' seconds'])
toco=toc;

if p.debug==9, keyboard, elseif p.debug>1, pause, end

%
% call thrid script RESOLVE to perform the inversion
%
disp(' PRESOLVE finished, call RESOLVE ')
resolve
