%%
% RESOLVE assumes that PRESOLVE ran and has prepared data for inversion
%
% Main invers solution of
%  1) top - bottom velocity profile
%  2) bottom reference profile (if bottom track available)
%  3) down-up cast solution only
% 
%  compute shear based solution
%
%  compute vertical diffusivity (experimental)
%
%  save all data in various formats
%

toco=toc;

% 
ps=setdefv(ps,'shear',1);

% 
%  take advantage of presolve if it existed
%  call the main inversion routine
%

if exist('dr')
 [dr,ps,de]=getinv(di,p,ps,dr,1);
else
 [dr,ps,de]=getinv(di,p,ps);
end
disp(['==> inversion took ',int2str(toc-toco),' seconds'])
toco=toc;

if p.debug==9, keyboard, elseif p.debug>1, pause, end

%
% check inversion constraints
% 
p=checkinv(dr,di,de,der,p,ps);
if p.debug==9, keyboard, elseif p.debug>1, pause, end

if existf(de,'bvel')
 p=checkbtrk(d,di,de,dr,p);
 if p.debug==9, keyboard, elseif p.debug>1, pause, end
end

% Compute 'old fashioned' shear based solution 
%  two choices, fisrt us all data
%  second use super ensemble data
if ps.shear>0
 if ps.shear==1
  [ds,dr,ps,p]=getshear2(d,p,ps,dr);
 else
  [ds,dr,ps,p]=getshear2(di,p,ps,dr);
 end
 if p.debug==9, keyboard, elseif p.debug>1, pause, end
end

% 
%  prepare ctd data for output
%
if existf(d,'ctdprof_p')
 % first save CTD data with profile
 dr.ctd_t=interp1q(d.ctdprof_z,d.ctdprof_t,dr.z);
 dr.ctd_s=interp1q(d.ctdprof_z,d.ctdprof_s,dr.z);
end
if existf(d,'ctdprof_ss')
 dr.ctd_ss=interp1q(d.ctdprof_z,d.ctdprof_ss,dr.z);
end

%
% estimate vertical diffusivity if CTD N2 profile exist
%  this is totally experimental and might be WRONG!
%
if existf(d,'ctdprof_N2') & exist('pk')
 disp(' compute vertical diffusivity')
 dr.ctd_N2=interp1q(d.ctdprof_z,d.ctdprof_N2,dr.z);
 [dk,pk]=getkzprof(dr,pk,dr.ctd_N2);
 if existf(dk,'z')
  if length(dk.z)>1
   zz=[dk.z(1)-dk.zrange/4,dk.z,dk.z(end)+dk.zrange/4]';
   Kz=dk.Kz([1 1:end end])';
   dr.Kz=interp1q(zz,Kz,dr.z);
  end
 end
 if p.debug==9, keyboard, elseif p.debug>1, pause, end
end

% Convert p.warn to one line of text with newline characters
p.warnings = [];
for i = 1:size(p.warnp,1)
   p.warnings = [p.warnings deblank(p.warnp(i,:)) char(10)];
end

% Plot final results
 figure(1), clf
 plotinv(dr,d,p,ps)
 if p.debug==9, keyboard, elseif p.debug>1, pause, end

figure(11)
clf
% experimental diagnostic of battery voltage
%
p=battery(p);

%
% complete task by repeating the most important warnings
%
if size(p.warn,1)>1
 disp(p.warn)
 disp(' ')
 disp(p.warnp)
 for j=1:size(p.warn,1)
  text(0,1.1-j/10,p.warn(j,:),'color','r','fontsize',14,'fontweight','bold')
 end
 axis off
else
 text(0,1.1-1/10,'LADCP profile OK','color','g','fontsize',30,'fontweight','bold')
 axis off
end

streamer([p.name,' Figure 11']);
pause(0.01)



% OUTPUT to files
if length(f.res)>1

  % save results to ASCII, MATLAB and NETCD files
  disp(' save results ')
  saveres(dr,p,ps,f)
  da=savearch(dr,d,p,ps,f,att);
  if p.debug==9, keyboard, elseif p.debug>1, pause, end

  
  % diary off 
  diary off

  % save a protocol
  saveprot
  if p.debug==9, keyboard, elseif p.debug>1, pause, end

end

%
% save plots to postscipt files 
%
for i = 1:length(p.saveplot)
   j=p.saveplot(i);
   figure(j)
   eval(['print -dpsc ',f.res,'_' int2str(j) '.ps '])
end

for i = 1:length(p.saveplot_png)
   j=p.saveplot_png(i);
   figure(j)
   eval(['print -dpng ',f.res,'_' int2str(j) '.png '])
end

% 
% finally write summary of processing to single window for LADCP 
% operator
%
% 

%  close all files in case we have been tardy earlier
fclose('all');
disp(' ')
disp(['==> The whole task took ',int2str(toc),' seconds'])

%
% END of RESOLVE
%
